
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    print qq{1..0 # SKIP these tests are for testing by the author\n};
    exit
  }
}

use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::NoTabs 0.15

use Test::More 0.88;
use Test::NoTabs;

my @files = (
    'bin/cosine_cmp',
    'bin/minhash_cmp',
    'bin/uniq_wc',
    'lib/Text/SpeedyFx.pm',
    't/00-compile.t',
    't/00-load.t',
    't/05-nedtrie.t',
    't/10-tiny.t',
    't/20-small.t',
    't/30-latin1.t',
    't/40-leak.t',
    't/50-break.t',
    't/author-critic.t',
    't/author-distmeta.t',
    't/author-eol.t',
    't/author-minimum-version.t',
    't/author-mojibake.t',
    't/author-no-tabs.t',
    't/author-pod-coverage.t',
    't/author-pod-linkcheck.t',
    't/author-pod-syntax.t',
    't/author-portability.t',
    't/author-synopsis.t',
    't/author-test-version.t',
    't/release-cpan-changes.t',
    't/release-dist-manifest.t',
    't/release-kwalitee.t',
    't/release-unused-vars.t'
);

notabs_ok($_) foreach @files;
done_testing;
